@@ -26,31 +26,6 @@
 #include <cups/language.h>
 
 
-/*
- * CUPS printing interface definitions...
- */
-
-static int cups_job_delete(int snum, struct printjob *pjob);
-static int cups_job_pause(int snum, struct printjob *pjob);
-static int cups_job_resume(int snum, struct printjob *pjob);
-static int cups_job_submit(int snum, struct printjob *pjob);
-static int cups_queue_get(int snum, print_queue_struct **q,
-                          print_status_struct *status);
-static int cups_queue_pause(int snum);
-static int cups_queue_resume(int snum);
-
-
-struct printif	cups_printif =
-		{
-		  cups_queue_get,
-		  cups_queue_pause,
-		  cups_queue_resume,
-		  cups_job_delete,
-		  cups_job_pause,
-		  cups_job_resume,
-		  cups_job_submit,
-		};
-
 /*
  * 'cups_passwd_cb()' - The CUPS password callback...
  */
@@ -58,39 +33,42 @@ struct printif	cups_printif =
 static const char *				/* O - Password or NULL */
 cups_passwd_cb(const char *prompt)	/* I - Prompt */
 {
- /*
-  * Always return NULL to indicate that no password is available...
-  */
+	/*
+	 * Always return NULL to indicate that no password is available...
+	 */
 
-  return (NULL);
+	return (NULL);
 }
 
-
-/*
- * 'cups_printer_fn()' - Call a function for every printer known to the
- *                       system.
- */
-
-void cups_printer_fn(void (*fn)(char *, char *))
+static const char *cups_server(void)
 {
-	/* I - Function to call */
-	http_t		*http;		/* HTTP connection to server */
-	ipp_t		*request,	/* IPP Request */
-			*response;	/* IPP Response */
+	if ((lp_cups_server() != NULL) && (strlen(lp_cups_server()) > 0)) {
+		DEBUG(10, ("cups server explicitly set to %s\n",
+			   lp_cups_server()));
+		return lp_cups_server();
+	}
+
+	DEBUG(10, ("cups server left to default %s\n", cupsServer()));
+	return cupsServer();
+}
+
+BOOL cups_cache_reload(void)
+{
+	http_t		*http = NULL;		/* HTTP connection to server */
+	ipp_t		*request = NULL,	/* IPP Request */
+			*response = NULL;	/* IPP Response */
 	ipp_attribute_t	*attr;		/* Current attribute */
-	cups_lang_t	*language;	/* Default language */
+	cups_lang_t	*language = NULL;	/* Default language */
 	char		*name,		/* printer-name attribute */
-			*make_model,	/* printer-make-and-model attribute */
 			*info;		/* printer-info attribute */
 	static const char *requested[] =/* Requested attributes */
 			{
 			  "printer-name",
-			  "printer-make-and-model",
 			  "printer-info"
 			};       
+	BOOL ret = False;
 
-
-	DEBUG(5,("cups_printer_fn(%p)\n", fn));
+	DEBUG(5, ("reloading cups printcap cache\n"));
 
        /*
         * Make sure we don't ask for passwords...
@@ -102,11 +80,10 @@ void cups_printer_fn(void (*fn)(char *, char *))
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cupsServer(), ippPort())) == NULL)
-	{
+	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
 		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cupsServer(), strerror(errno)));
-		return;
+			 cups_server(), strerror(errno)));
+		goto out;
 	}
 
        /*
@@ -140,16 +117,13 @@ void cups_printer_fn(void (*fn)(char *, char *))
 	* Do the request and get back a response...
 	*/
 
-	if ((response = cupsDoRequest(http, request, "/")) == NULL)
-	{
+	if ((response = cupsDoRequest(http, request, "/")) == NULL) {
 		DEBUG(0,("Unable to get printer list - %s\n",
 			 ippErrorString(cupsLastError())));
-		httpClose(http);
-		return;
+		goto out;
 	}
 
-	for (attr = response->attrs; attr != NULL;)
-	{
+	for (attr = response->attrs; attr != NULL;) {
 	       /*
 		* Skip leading attributes until we hit a printer...
 		*/
@@ -165,19 +139,13 @@ void cups_printer_fn(void (*fn)(char *, char *))
 		*/
 
 		name       = NULL;
-		make_model = NULL;
 		info       = NULL;
 
-		while (attr != NULL && attr->group_tag == IPP_TAG_PRINTER)
-		{
+		while (attr != NULL && attr->group_tag == IPP_TAG_PRINTER) {
         		if (strcmp(attr->name, "printer-name") == 0 &&
 			    attr->value_tag == IPP_TAG_NAME)
 				name = attr->values[0].string.text;
 
-        		if (strcmp(attr->name, "printer-make-and-model") == 0 &&
-			    attr->value_tag == IPP_TAG_TEXT)
-				make_model = attr->values[0].string.text;
-
         		if (strcmp(attr->name, "printer-info") == 0 &&
 			    attr->value_tag == IPP_TAG_TEXT)
 				info = attr->values[0].string.text;
@@ -192,16 +160,13 @@ void cups_printer_fn(void (*fn)(char *, char *))
 		if (name == NULL)
 			break;
 
- 		if (info == NULL || !info[0])
-			(*fn)(name, make_model);
-		else
-			(*fn)(name, info);
-		
-
+		if (!pcap_cache_add(name, info)) {
+			goto out;
+		}
 	}
 
 	ippDelete(response);
-
+	response = NULL;
 
        /*
 	* Build a CUPS_GET_CLASSES request, which requires the following
@@ -217,8 +182,6 @@ void cups_printer_fn(void (*fn)(char *, char *))
 	request->request.op.operation_id = CUPS_GET_CLASSES;
 	request->request.op.request_id   = 1;
 
-	language = cupsLangDefault();
-
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
                      "attributes-charset", NULL, cupsLangEncoding(language));
 
@@ -234,16 +197,13 @@ void cups_printer_fn(void (*fn)(char *, char *))
 	* Do the request and get back a response...
 	*/
 
-	if ((response = cupsDoRequest(http, request, "/")) == NULL)
-	{
+	if ((response = cupsDoRequest(http, request, "/")) == NULL) {
 		DEBUG(0,("Unable to get printer list - %s\n",
 			 ippErrorString(cupsLastError())));
-		httpClose(http);
-		return;
+		goto out;
 	}
 
-	for (attr = response->attrs; attr != NULL;)
-	{
+	for (attr = response->attrs; attr != NULL;) {
 	       /*
 		* Skip leading attributes until we hit a printer...
 		*/
@@ -259,19 +219,13 @@ void cups_printer_fn(void (*fn)(char *, char *))
 		*/
 
 		name       = NULL;
-		make_model = NULL;
 		info       = NULL;
 
-		while (attr != NULL && attr->group_tag == IPP_TAG_PRINTER)
-		{
+		while (attr != NULL && attr->group_tag == IPP_TAG_PRINTER) {
         		if (strcmp(attr->name, "printer-name") == 0 &&
 			    attr->value_tag == IPP_TAG_NAME)
 				name = attr->values[0].string.text;
 
-        		if (strcmp(attr->name, "printer-make-and-model") == 0 &&
-			    attr->value_tag == IPP_TAG_TEXT)
-				make_model = attr->values[0].string.text;
-
         		if (strcmp(attr->name, "printer-info") == 0 &&
 			    attr->value_tag == IPP_TAG_TEXT)
 				info = attr->values[0].string.text;
@@ -286,115 +240,24 @@ void cups_printer_fn(void (*fn)(char *, char *))
 		if (name == NULL)
 			break;
 
- 		if (info == NULL || !info[0])
-			(*fn)(name, make_model);
-		else
-			(*fn)(name, info);
-		
-
+		if (!pcap_cache_add(name, info)) {
+			goto out;
+		}
 	}
 
-	ippDelete(response);
+	ret = True;
 
-       /*
-        * Close the connection to the server...
-	*/
+ out:
+	if (response)
+		ippDelete(response);
 
-	httpClose(http);
-}
+	if (language)
+		cupsLangFree(language);
 
-
-/*
- * 'cups_printername_ok()' - Provide the equivalent of pcap_printername_ok()
- *                           for CUPS.
- * O - 1 if printer name OK
- * I - Name of printer 
- */
-int cups_printername_ok(const char *name)
-{
-	http_t		*http;		/* HTTP connection to server */
-	ipp_t		*request,	/* IPP Request */
-			*response;	/* IPP Response */
-	cups_lang_t	*language;	/* Default language */
-	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
-
-
-	DEBUG(5,("cups_printername_ok(\"%s\")\n", name));
-
-       /*
-        * Make sure we don't ask for passwords...
-	*/
-
-        cupsSetPasswordCB(cups_passwd_cb);
-
-       /*
-	* Try to connect to the server...
-	*/
-
-	if ((http = httpConnect(cupsServer(), ippPort())) == NULL)
-	{
-		DEBUG(3,("Unable to connect to CUPS server %s - %s\n", 
-			 cupsServer(), strerror(errno)));
-		return (0);
-	}
-
-       /*
-	* Build an IPP_GET_PRINTER_ATTRS request, which requires the following
-	* attributes:
-	*
-	*    attributes-charset
-	*    attributes-natural-language
-	*    requested-attributes
-	*    printer-uri
-	*/
-
-	request = ippNew();
-
-	request->request.op.operation_id = IPP_GET_PRINTER_ATTRIBUTES;
-	request->request.op.request_id   = 1;
-
-	language = cupsLangDefault();
-
-	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
-                     "attributes-charset", NULL, cupsLangEncoding(language));
-
-	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
-                     "attributes-natural-language", NULL, language->language);
-
-	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
-                     "requested-attributes", NULL, "printer-uri");
-
-	slprintf(uri, sizeof(uri) - 1, "ipp://localhost/printers/%s", name);
-
-	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
-                     "printer-uri", NULL, uri);
-
-       /*
-	* Do the request and get back a response...
-	*/
-
-	if ((response = cupsDoRequest(http, request, "/")) == NULL)
-	{
-		DEBUG(3,("Unable to get printer status for %s - %s\n", name,
-			 ippErrorString(cupsLastError())));
+	if (http)
 		httpClose(http);
-		return (0);
-	}
 
-	httpClose(http);
-
-	if (response->request.status.status_code >= IPP_OK_CONFLICT)
-	{
-		DEBUG(3,("Unable to get printer status for %s - %s\n", name,
-			 ippErrorString(response->request.status.status_code)));
-		ippDelete(response);
-		return (0);
-	}
-	else
-	{
-		ippDelete(response);
-		return (1);
-	}
+	return ret;
 }
 
 
@@ -402,14 +265,13 @@ int cups_printername_ok(const char *name)
  * 'cups_job_delete()' - Delete a job.
  */
 
-static int
-cups_job_delete(int snum, struct printjob *pjob)
+static int cups_job_delete(int snum, struct printjob *pjob)
 {
-	int		ret;		/* Return value */
-	http_t		*http;		/* HTTP connection to server */
-	ipp_t		*request,	/* IPP Request */
-			*response;	/* IPP Response */
-	cups_lang_t	*language;	/* Default language */
+	int		ret = 1;		/* Return value */
+	http_t		*http = NULL;		/* HTTP connection to server */
+	ipp_t		*request = NULL,	/* IPP Request */
+			*response = NULL;	/* IPP Response */
+	cups_lang_t	*language = NULL;	/* Default language */
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
 
 
@@ -425,11 +287,10 @@ cups_job_delete(int snum, struct printjob *pjob)
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cupsServer(), ippPort())) == NULL)
-	{
+	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
 		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cupsServer(), strerror(errno)));
-		return (1);
+			 cups_server(), strerror(errno)));
+		goto out;
 	}
 
        /*
@@ -466,25 +327,29 @@ cups_job_delete(int snum, struct printjob *pjob)
 	* Do the request and get back a response...
 	*/
 
-        ret = 1;
-
-	if ((response = cupsDoRequest(http, request, "/jobs")) != NULL)
-	{
-	  if (response->request.status.status_code >= IPP_OK_CONFLICT)
+	if ((response = cupsDoRequest(http, request, "/jobs")) != NULL) {
+		if (response->request.status.status_code >= IPP_OK_CONFLICT) {
+			DEBUG(0,("Unable to cancel job %d - %s\n", pjob->sysjob,
+				ippErrorString(cupsLastError())));
+		} else {
+			ret = 0;
+		}
+	} else {
 		DEBUG(0,("Unable to cancel job %d - %s\n", pjob->sysjob,
-			 ippErrorString(cupsLastError())));
-          else
-	  	ret = 0;
-
-	  ippDelete(response);
+			ippErrorString(cupsLastError())));
 	}
-	else
-	  DEBUG(0,("Unable to cancel job %d - %s\n", pjob->sysjob,
-		   ippErrorString(cupsLastError())));
 
-	httpClose(http);
+ out:
+	if (response)
+		ippDelete(response);
 
-	return (ret);
+	if (language)
+		cupsLangFree(language);
+
+	if (http)
+		httpClose(http);
+
+	return ret;
 }
 
 
@@ -492,14 +357,13 @@ cups_job_delete(int snum, struct printjob *pjob)
  * 'cups_job_pause()' - Pause a job.
  */
 
-static int
-cups_job_pause(int snum, struct printjob *pjob)
+static int cups_job_pause(int snum, struct printjob *pjob)
 {
-	int		ret;		/* Return value */
-	http_t		*http;		/* HTTP connection to server */
-	ipp_t		*request,	/* IPP Request */
-			*response;	/* IPP Response */
-	cups_lang_t	*language;	/* Default language */
+	int		ret = 1;		/* Return value */
+	http_t		*http = NULL;		/* HTTP connection to server */
+	ipp_t		*request = NULL,	/* IPP Request */
+			*response = NULL;	/* IPP Response */
+	cups_lang_t	*language = NULL;	/* Default language */
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
 
 
@@ -515,11 +379,10 @@ cups_job_pause(int snum, struct printjob *pjob)
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cupsServer(), ippPort())) == NULL)
-	{
+	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
 		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cupsServer(), strerror(errno)));
-		return (1);
+			 cups_server(), strerror(errno)));
+		goto out;
 	}
 
        /*
@@ -556,25 +419,29 @@ cups_job_pause(int snum, struct printjob *pjob)
 	* Do the request and get back a response...
 	*/
 
-        ret = 1;
-
-	if ((response = cupsDoRequest(http, request, "/jobs")) != NULL)
-	{
-	  if (response->request.status.status_code >= IPP_OK_CONFLICT)
+	if ((response = cupsDoRequest(http, request, "/jobs")) != NULL) {
+		if (response->request.status.status_code >= IPP_OK_CONFLICT) {
+			DEBUG(0,("Unable to hold job %d - %s\n", pjob->sysjob,
+				ippErrorString(cupsLastError())));
+		} else {
+			ret = 0;
+		}
+	} else {
 		DEBUG(0,("Unable to hold job %d - %s\n", pjob->sysjob,
-			 ippErrorString(cupsLastError())));
-          else
-	  	ret = 0;
-
-	  ippDelete(response);
+			ippErrorString(cupsLastError())));
 	}
-	else
-	  DEBUG(0,("Unable to hold job %d - %s\n", pjob->sysjob,
-		   ippErrorString(cupsLastError())));
 
-	httpClose(http);
+ out:
+	if (response)
+		ippDelete(response);
 
-	return (ret);
+	if (language)
+		cupsLangFree(language);
+
+	if (http)
+		httpClose(http);
+
+	return ret;
 }
 
 
@@ -582,14 +449,13 @@ cups_job_pause(int snum, struct printjob *pjob)
  * 'cups_job_resume()' - Resume a paused job.
  */
 
-static int
-cups_job_resume(int snum, struct printjob *pjob)
+static int cups_job_resume(int snum, struct printjob *pjob)
 {
-	int		ret;		/* Return value */
-	http_t		*http;		/* HTTP connection to server */
-	ipp_t		*request,	/* IPP Request */
-			*response;	/* IPP Response */
-	cups_lang_t	*language;	/* Default language */
+	int		ret = 1;		/* Return value */
+	http_t		*http = NULL;		/* HTTP connection to server */
+	ipp_t		*request = NULL,	/* IPP Request */
+			*response = NULL;	/* IPP Response */
+	cups_lang_t	*language = NULL;	/* Default language */
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
 
 
@@ -605,11 +471,10 @@ cups_job_resume(int snum, struct printjob *pjob)
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cupsServer(), ippPort())) == NULL)
-	{
+	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
 		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cupsServer(), strerror(errno)));
-		return (1);
+			 cups_server(), strerror(errno)));
+		goto out;
 	}
 
        /*
@@ -646,25 +511,29 @@ cups_job_resume(int snum, struct printjob *pjob)
 	* Do the request and get back a response...
 	*/
 
-        ret = 1;
-
-	if ((response = cupsDoRequest(http, request, "/jobs")) != NULL)
-	{
-	  if (response->request.status.status_code >= IPP_OK_CONFLICT)
+	if ((response = cupsDoRequest(http, request, "/jobs")) != NULL) {
+		if (response->request.status.status_code >= IPP_OK_CONFLICT) {
+			DEBUG(0,("Unable to release job %d - %s\n", pjob->sysjob,
+				ippErrorString(cupsLastError())));
+		} else {
+			ret = 0;
+		}
+	} else {
 		DEBUG(0,("Unable to release job %d - %s\n", pjob->sysjob,
-			 ippErrorString(cupsLastError())));
-          else
-	  	ret = 0;
-
-	  ippDelete(response);
+			ippErrorString(cupsLastError())));
 	}
-	else
-	  DEBUG(0,("Unable to release job %d - %s\n", pjob->sysjob,
-		   ippErrorString(cupsLastError())));
 
-	httpClose(http);
+ out:
+	if (response)
+		ippDelete(response);
 
-	return (ret);
+	if (language)
+		cupsLangFree(language);
+
+	if (http)
+		httpClose(http);
+
+	return ret;
 }
 
 
@@ -672,19 +541,18 @@ cups_job_resume(int snum, struct printjob *pjob)
  * 'cups_job_submit()' - Submit a job for printing.
  */
 
-static int
-cups_job_submit(int snum, struct printjob *pjob)
+static int cups_job_submit(int snum, struct printjob *pjob)
 {
-	int		ret;		/* Return value */
-	http_t		*http;		/* HTTP connection to server */
-	ipp_t		*request,	/* IPP Request */
-			*response;	/* IPP Response */
-	cups_lang_t	*language;	/* Default language */
+	int		ret = 1;		/* Return value */
+	http_t		*http = NULL;		/* HTTP connection to server */
+	ipp_t		*request = NULL,	/* IPP Request */
+			*response = NULL;	/* IPP Response */
+	cups_lang_t	*language = NULL;	/* Default language */
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
-	char 		*clientname; 	/* hostname of client for job-originating-host attribute */
+	char 		*clientname = NULL; 	/* hostname of client for job-originating-host attribute */
 	pstring		new_jobname;
 	int		num_options = 0; 
-	cups_option_t 	*options;
+	cups_option_t 	*options = NULL;
 
 	DEBUG(5,("cups_job_submit(%d, %p (%d))\n", snum, pjob, pjob->sysjob));
 
@@ -698,11 +566,10 @@ cups_job_submit(int snum, struct printjob *pjob)
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cupsServer(), ippPort())) == NULL)
-	{
+	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
 		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cupsServer(), strerror(errno)));
-		return (1);
+			 cups_server(), strerror(errno)));
+		goto out;
 	}
 
        /*
@@ -770,47 +637,55 @@ cups_job_submit(int snum, struct printjob *pjob)
 
 	slprintf(uri, sizeof(uri) - 1, "/printers/%s", PRINTERNAME(snum));
 
-        ret = 1;
-	if ((response = cupsDoFileRequest(http, request, uri,
-	                                  pjob->filename)) != NULL)
-	{
-		if (response->request.status.status_code >= IPP_OK_CONFLICT)
+	if ((response = cupsDoFileRequest(http, request, uri, pjob->filename)) != NULL) {
+		if (response->request.status.status_code >= IPP_OK_CONFLICT) {
 			DEBUG(0,("Unable to print file to %s - %s\n", PRINTERNAME(snum),
 			         ippErrorString(cupsLastError())));
-        	else
+		} else {
 			ret = 0;
-
-		ippDelete(response);
-	}
-	else
+		}
+	} else {
 		DEBUG(0,("Unable to print file to `%s' - %s\n", PRINTERNAME(snum),
 			 ippErrorString(cupsLastError())));
-
-	httpClose(http);
+	}
 
 	if ( ret == 0 )
 		unlink(pjob->filename);
 	/* else print_job_end will do it for us */
 
-	return (ret);
+ out:
+	if (response)
+		ippDelete(response);
+
+	if (language)
+		cupsLangFree(language);
+
+	if (http)
+		httpClose(http);
+
+	return ret;
 }
 
 /*
  * 'cups_queue_get()' - Get all the jobs in the print queue.
  */
 
-static int
-cups_queue_get(int snum, print_queue_struct **q, print_status_struct *status)
+static int cups_queue_get(const char *sharename,
+               enum printing_types printing_type,
+               char *lpq_command,
+               print_queue_struct **q, 
+               print_status_struct *status)
 {
-	http_t		*http;		/* HTTP connection to server */
-	ipp_t		*request,	/* IPP Request */
-			*response;	/* IPP Response */
-	ipp_attribute_t	*attr;		/* Current attribute */
-	cups_lang_t	*language;	/* Default language */
+	fstring		printername;
+	http_t		*http = NULL;		/* HTTP connection to server */
+	ipp_t		*request = NULL,	/* IPP Request */
+			*response = NULL;	/* IPP Response */
+	ipp_attribute_t	*attr = NULL;		/* Current attribute */
+	cups_lang_t	*language = NULL;	/* Default language */
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
-	int		qcount,		/* Number of active queue entries */
-			qalloc;		/* Number of queue entries allocated */
-	print_queue_struct *queue,	/* Queue entries */
+	int		qcount = 0,		/* Number of active queue entries */
+			qalloc = 0;		/* Number of queue entries allocated */
+	print_queue_struct *queue = NULL,	/* Queue entries */
 			*temp;		/* Temporary pointer for queue */
 	const char	*user_name,	/* job-originating-user-name attribute */
 			*job_name;	/* job-name attribute */
@@ -835,8 +710,17 @@ cups_queue_get(int snum, print_queue_struct **q, print_status_struct *status)
 			  "printer-state-message"
 			};
 
+	*q = NULL;
 
-	DEBUG(5,("cups_queue_get(%d, %p, %p)\n", snum, q, status));
+	/* HACK ALERT!!!  The porblem with support the 'printer name' 
+	   option is that we key the tdb off the sharename.  So we will 
+	   overload the lpq_command string to pass in the printername 
+	   (which is basically what we do for non-cups printers ... using 
+	   the lpq_command to get the queue listing). */
+
+	fstrcpy( printername, lpq_command );
+
+	DEBUG(5,("cups_queue_get(%s, %p, %p)\n", printername, q, status));
 
        /*
         * Make sure we don't ask for passwords...
@@ -848,19 +732,17 @@ cups_queue_get(int snum, print_queue_struct **q, print_status_struct *status)
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cupsServer(), ippPort())) == NULL)
-	{
+	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
 		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cupsServer(), strerror(errno)));
-		return (0);
+			 cups_server(), strerror(errno)));
+		goto out;
 	}
 
        /*
         * Generate the printer URI...
 	*/
 
-	slprintf(uri, sizeof(uri) - 1, "ipp://localhost/printers/%s",
-	         PRINTERNAME(snum));
+	slprintf(uri, sizeof(uri) - 1, "ipp://localhost/printers/%s", printername);
 
        /*
 	* Build an IPP_GET_JOBS request, which requires the following
@@ -897,22 +779,16 @@ cups_queue_get(int snum, print_queue_struct **q, print_status_struct *status)
 	* Do the request and get back a response...
 	*/
 
-	if ((response = cupsDoRequest(http, request, "/")) == NULL)
-	{
+	if ((response = cupsDoRequest(http, request, "/")) == NULL) {
 		DEBUG(0,("Unable to get jobs for %s - %s\n", uri,
 			 ippErrorString(cupsLastError())));
-		httpClose(http);
-		return (0);
+		goto out;
 	}
 
-	if (response->request.status.status_code >= IPP_OK_CONFLICT)
-	{
+	if (response->request.status.status_code >= IPP_OK_CONFLICT) {
 		DEBUG(0,("Unable to get jobs for %s - %s\n", uri,
 			 ippErrorString(response->request.status.status_code)));
-		ippDelete(response);
-		httpClose(http);
-
-		return (0);
+		goto out;
 	}
 
        /*
@@ -923,8 +799,7 @@ cups_queue_get(int snum, print_queue_struct **q, print_status_struct *status)
 	qalloc = 0;
 	queue  = NULL;
 
-        for (attr = response->attrs; attr != NULL; attr = attr->next)
-	{
+        for (attr = response->attrs; attr != NULL; attr = attr->next) {
 	       /*
 		* Skip leading attributes until we hit a job...
 		*/
@@ -938,20 +813,16 @@ cups_queue_get(int snum, print_queue_struct **q, print_status_struct *status)
 	       /*
 	        * Allocate memory as needed...
 		*/
-		if (qcount >= qalloc)
-		{
+		if (qcount >= qalloc) {
 			qalloc += 16;
 
-			temp = Realloc(queue, sizeof(print_queue_struct) * qalloc);
+			temp = SMB_REALLOC_ARRAY(queue, print_queue_struct, qalloc);
 
-			if (temp == NULL)
-			{
+			if (temp == NULL) {
 				DEBUG(0,("cups_queue_get: Not enough memory!"));
-				ippDelete(response);
-				httpClose(http);
-
+				qcount = 0;
 				SAFE_FREE(queue);
-				return (0);
+				goto out;
 			}
 
 			queue = temp;
@@ -972,10 +843,8 @@ cups_queue_get(int snum, print_queue_struct **q, print_status_struct *status)
 		user_name    = NULL;
 		job_name     = NULL;
 
-		while (attr != NULL && attr->group_tag == IPP_TAG_JOB)
-		{
-        		if (attr->name == NULL)
-			{
+		while (attr != NULL && attr->group_tag == IPP_TAG_JOB) {
+        		if (attr->name == NULL) {
 				attr = attr->next;
 				break;
 			}
@@ -1015,12 +884,11 @@ cups_queue_get(int snum, print_queue_struct **q, print_status_struct *status)
 		* See if we have everything needed...
 		*/
 
-		if (user_name == NULL || job_name == NULL || job_id == 0)
-		{
-        	  if (attr == NULL)
-		    break;
-		  else
-        	    continue;
+		if (user_name == NULL || job_name == NULL || job_id == 0) {
+			if (attr == NULL)
+				break;
+			else
+				continue;
 		}
 
 		temp->job      = job_id;
@@ -1037,10 +905,11 @@ cups_queue_get(int snum, print_queue_struct **q, print_status_struct *status)
 		qcount ++;
 
 		if (attr == NULL)
-        	  break;
+			break;
 	}
 
 	ippDelete(response);
+	response = NULL;
 
        /*
 	* Build an IPP_GET_PRINTER_ATTRIBUTES request, which requires the
@@ -1057,8 +926,6 @@ cups_queue_get(int snum, print_queue_struct **q, print_status_struct *status)
 	request->request.op.operation_id = IPP_GET_PRINTER_ATTRIBUTES;
 	request->request.op.request_id   = 1;
 
-	language = cupsLangDefault();
-
 	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
                      "attributes-charset", NULL, cupsLangEncoding(language));
 
@@ -1077,31 +944,25 @@ cups_queue_get(int snum, print_queue_struct **q, print_status_struct *status)
 	* Do the request and get back a response...
 	*/
 
-	if ((response = cupsDoRequest(http, request, "/")) == NULL)
-	{
-		DEBUG(0,("Unable to get printer status for %s - %s\n", PRINTERNAME(snum),
+	if ((response = cupsDoRequest(http, request, "/")) == NULL) {
+		DEBUG(0,("Unable to get printer status for %s - %s\n", printername,
 			 ippErrorString(cupsLastError())));
-		httpClose(http);
 		*q = queue;
-		return (qcount);
+		goto out;
 	}
 
-	if (response->request.status.status_code >= IPP_OK_CONFLICT)
-	{
-		DEBUG(0,("Unable to get printer status for %s - %s\n", PRINTERNAME(snum),
+	if (response->request.status.status_code >= IPP_OK_CONFLICT) {
+		DEBUG(0,("Unable to get printer status for %s - %s\n", printername,
 			 ippErrorString(response->request.status.status_code)));
-		ippDelete(response);
-		httpClose(http);
 		*q = queue;
-		return (qcount);
+		goto out;
 	}
 
        /*
         * Get the current printer status and convert it to the SAMBA values.
 	*/
 
-        if ((attr = ippFindAttribute(response, "printer-state", IPP_TAG_ENUM)) != NULL)
-	{
+        if ((attr = ippFindAttribute(response, "printer-state", IPP_TAG_ENUM)) != NULL) {
 		if (attr->values[0].integer == IPP_PRINTER_STOPPED)
 			status->status = LPSTAT_STOPPED;
 		else
@@ -1112,16 +973,23 @@ cups_queue_get(int snum, print_queue_struct **q, print_status_struct *status)
 	                             IPP_TAG_TEXT)) != NULL)
 	        fstrcpy(status->message, attr->values[0].string.text);
 
-        ippDelete(response);
-
        /*
         * Return the job queue...
 	*/
 
-	httpClose(http);
-
 	*q = queue;
-	return (qcount);
+
+ out:
+	if (response)
+		ippDelete(response);
+
+	if (language)
+		cupsLangFree(language);
+
+	if (http)
+		httpClose(http);
+
+	return qcount;
 }
 
 
@@ -1129,15 +997,14 @@ cups_queue_get(int snum, print_queue_struct **q, print_status_struct *status)
  * 'cups_queue_pause()' - Pause a print queue.
  */
 
-static int
-cups_queue_pause(int snum)
+static int cups_queue_pause(int snum)
 {
 	extern userdom_struct current_user_info;
-	int		ret;		/* Return value */
-	http_t		*http;		/* HTTP connection to server */
-	ipp_t		*request,	/* IPP Request */
-			*response;	/* IPP Response */
-	cups_lang_t	*language;	/* Default language */
+	int		ret = 1;		/* Return value */
+	http_t		*http = NULL;		/* HTTP connection to server */
+	ipp_t		*request = NULL,	/* IPP Request */
+			*response = NULL;	/* IPP Response */
+	cups_lang_t	*language = NULL;	/* Default language */
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
 
 
@@ -1153,11 +1020,10 @@ cups_queue_pause(int snum)
 	 * Try to connect to the server...
 	 */
 
-	if ((http = httpConnect(cupsServer(), ippPort())) == NULL)
-	{
+	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
 		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cupsServer(), strerror(errno)));
-		return (1);
+			 cups_server(), strerror(errno)));
+		goto out;
 	}
 
 	/*
@@ -1195,25 +1061,29 @@ cups_queue_pause(int snum)
 	* Do the request and get back a response...
 	*/
 
-        ret = 1;
-
-	if ((response = cupsDoRequest(http, request, "/admin/")) != NULL)
-	{
-	  if (response->request.status.status_code >= IPP_OK_CONFLICT)
+	if ((response = cupsDoRequest(http, request, "/admin/")) != NULL) {
+		if (response->request.status.status_code >= IPP_OK_CONFLICT) {
+			DEBUG(0,("Unable to pause printer %s - %s\n", PRINTERNAME(snum),
+				ippErrorString(cupsLastError())));
+		} else {
+			ret = 0;
+		}
+	} else {
 		DEBUG(0,("Unable to pause printer %s - %s\n", PRINTERNAME(snum),
-			 ippErrorString(cupsLastError())));
-          else
-	  	ret = 0;
-
-	  ippDelete(response);
+			ippErrorString(cupsLastError())));
 	}
-	else
-	  DEBUG(0,("Unable to pause printer %s - %s\n", PRINTERNAME(snum),
-		   ippErrorString(cupsLastError())));
 
-	httpClose(http);
+ out:
+	if (response)
+		ippDelete(response);
 
-	return (ret);
+	if (language)
+		cupsLangFree(language);
+
+	if (http)
+		httpClose(http);
+
+	return ret;
 }
 
 
@@ -1221,15 +1091,14 @@ cups_queue_pause(int snum)
  * 'cups_queue_resume()' - Restart a print queue.
  */
 
-static int
-cups_queue_resume(int snum)
+static int cups_queue_resume(int snum)
 {
 	extern userdom_struct current_user_info;
-	int		ret;		/* Return value */
-	http_t		*http;		/* HTTP connection to server */
-	ipp_t		*request,	/* IPP Request */
-			*response;	/* IPP Response */
-	cups_lang_t	*language;	/* Default language */
+	int		ret = 1;		/* Return value */
+	http_t		*http = NULL;		/* HTTP connection to server */
+	ipp_t		*request = NULL,	/* IPP Request */
+			*response = NULL;	/* IPP Response */
+	cups_lang_t	*language = NULL;	/* Default language */
 	char		uri[HTTP_MAX_URI]; /* printer-uri attribute */
 
 
@@ -1245,11 +1114,10 @@ cups_queue_resume(int snum)
 	* Try to connect to the server...
 	*/
 
-	if ((http = httpConnect(cupsServer(), ippPort())) == NULL)
-	{
+	if ((http = httpConnect(cups_server(), ippPort())) == NULL) {
 		DEBUG(0,("Unable to connect to CUPS server %s - %s\n", 
-			 cupsServer(), strerror(errno)));
-		return (1);
+			 cups_server(), strerror(errno)));
+		goto out;
 	}
 
        /*
@@ -1287,27 +1155,46 @@ cups_queue_resume(int snum)
 	* Do the request and get back a response...
 	*/
 
-        ret = 1;
-
-	if ((response = cupsDoRequest(http, request, "/admin/")) != NULL)
-	{
-	  if (response->request.status.status_code >= IPP_OK_CONFLICT)
+	if ((response = cupsDoRequest(http, request, "/admin/")) != NULL) {
+		if (response->request.status.status_code >= IPP_OK_CONFLICT) {
+			DEBUG(0,("Unable to resume printer %s - %s\n", PRINTERNAME(snum),
+				ippErrorString(cupsLastError())));
+		} else {
+			ret = 0;
+		}
+	} else {
 		DEBUG(0,("Unable to resume printer %s - %s\n", PRINTERNAME(snum),
-			 ippErrorString(cupsLastError())));
-          else
-	  	ret = 0;
-
-	  ippDelete(response);
+			ippErrorString(cupsLastError())));
 	}
-	else
-	  DEBUG(0,("Unable to resume printer %s - %s\n", PRINTERNAME(snum),
-		   ippErrorString(cupsLastError())));
 
-	httpClose(http);
+ out:
+	if (response)
+		ippDelete(response);
 
-	return (ret);
+	if (language)
+		cupsLangFree(language);
+
+	if (http)
+		httpClose(http);
+
+	return ret;
 }
 
+/*******************************************************************
+ * CUPS printing interface definitions...
+ ******************************************************************/
+
+struct printif	cups_printif =
+{
+	PRINT_CUPS,
+	cups_queue_get,
+	cups_queue_pause,
+	cups_queue_resume,
+	cups_job_delete,
+	cups_job_pause,
+	cups_job_resume,
+	cups_job_submit,
+};
 
 #else
  /* this keeps fussy compilers happy */
